<?php

namespace App\Http\Controllers;

use App\AdminOtherAccount;
use Illuminate\Http\Request;
use App\Branches;
use App\BranchPermission;
use App\Company;
use App\Expense;
use App\user;
use App\auth;
use App\Permission;

class BranchController extends Controller
{
    public function index()
    {
        $data['branches'] = Branches::all();
        return view('super_admin.branch_master.index')->with($data);
    }

    public function add() {
        $data['companies'] = Company::all();
        $data['permissions'] = Permission::all();
        return view('super_admin.branch_master.add')->with($data);
    }

    public function edit(Branches $branch)
    {
        $branch->load('permissions');
        $companies = Company::all();
        $permissions = Permission::all();
        return view('super_admin.branch_master.superadmin-edit', compact('branch', 'companies', 'permissions'));
    }

    public function view(Branches $branch)
    {
        $branch->load('permissions');
        $companies = Company::all();
        $permissions = Permission::all();
        return view('super_admin.branch_master.superadmin-view', compact('branch', 'companies', 'permissions'));
    }

    public function blockUnblock(Branches $branch)
    {
        try {
            $branch->update(['status'=>$branch->status==1?0:1]);
            flash()->addSuccess($branch->status==0?'Blocked':'Unblocked');
            return back();
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function insert(Request $request)
{
    $data = [
        'company_id' => $request->company_id,
        'country' => $request->country,
        'name' => $request->name,
        'currency' => $request->currency,
        'price' => $request->price,
        'software' => $request->software,
        'issue_date' => $request->issue_date,
        'expiry_date' => $request->expiry_date,
        'leniency' => $request->leniency,
        'verification' => $request->verification,
        'currency_id' => $request->currency_id,
        'currency_detail' => $request->currency_detail,
        'currency_decimal' => $request->currency_decimal,
        'minal_unit' => $request->minal_unit,
    ];

    $branch = Branches::create($data);

    $permissions = $request->input('permissions', []);
    $permissions[] = 128;

    if (!empty($permissions)) {
        foreach ($permissions as $permission) {
            BranchPermission::create(['branch_id' => $branch->id, 'permission_id' => $permission]);
        }
    }

    // Create VAT account
    $vatData = [
        'date' => now()->format('Y-m-d'),
        'account_name' => 'VAT',
        'class_of_account' => 'Tax & Duties',
        'description' => 'VAT account',
        'branch_id' => $branch->id,
    ];
    AdminOtherAccount::create($vatData);

    // Create Cash account
    $cashData = [
        'date' => now()->format('Y-m-d'),
        'account_name' => 'Cash',
        'class_of_account' => 'Cash In Hand',
        'description' => 'Cash In Hand',
        'branch_id' => $branch->id,
    ];
    AdminOtherAccount::create($cashData);

    // Create DISCOUNT account
    $discountData = [
        'date' => now()->format('Y-m-d'),
        'account_name' => 'DISCOUNT',
        'class_of_account_id' => 'Expenses',
        'description' => 'DISCOUNT account',
        'branch_id' => $branch->id,
    ];
    Expense::create($discountData);

    flash()->addSuccess('Record Created');
    return redirect('branch');
}


    public function update(Request $request, Branches $branch)
    {
        $branch->permissions()->delete();
        $branch->update($request->except(['permissions', '_token', '_method']));
        $permissions = $request->input('permissions', []); // Get the permissions array from the request
        $permissions[] = 128; // Add 128 to the array
        if (!empty($permissions)) {
            foreach ($permissions as $permission) {
                BranchPermission::create(['branch_id' => $branch->id, 'permission_id' => $permission]);
            }
        }
        return redirect('branch');
    }

    public function getCountries(Request $request){
        $country = Company::where('id',$request->company_id)->first();
        return $country;
    }

    function profile()
    {
        $branch = Branches::find(session()->get('selected_branch'));
        return view('crusher.crusher-profile', compact('branch'));
    }

    function profileUpdate()
    {
        try {
            if(!empty($address = request()->address)) Branches::find(session()->get('selected_branch'))->update(['address'=>$address]);
            if (request()->hasFile("image")) {
                $file = request()->file("image");
                $fileName = time().'.'.$file->getClientOriginalExtension();
                $filePath = $file->storeAs('uploads/user/images', $fileName, 'public');
                auth()->user()->update(['image'=>$filePath]);
            }
            if (request()->hasFile("company_logo")) {
                $file = request()->file("company_logo");
                $fileName = time().'.'.$file->getClientOriginalExtension();
                $filePath = $file->storeAs('uploads/company/images', $fileName, 'public');
                auth()->user()->update(['company_logo'=>$filePath]);
            }
            flash()->addSuccess('Updated');
            return back();
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return back();
        }
    }

}
