<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
use App\Company;

class CompanyController extends Controller
{

    protected $utils;

    public function __construct(Utils $utils)
    {
        $this->utils = $utils;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('company-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $companies = Company::Where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(25);
        return view('user.masters.companies.index', compact('companies'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('company-master')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.masters.companies.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $location_id = $this->getLocationId();
        $company = new Company();
        $company->location_id = $location_id;
        $company->name = $request->name;
        $company->email = $request->email;
        session()->get('branch')->address = $request->address;
        $company->phone = $request->phone;
        $company->mobile = $request->mobile;
        $company->whatsapp = $request->whatsapp;
        $company->vat_number = $request->vat_number;
        $company->trn_number = $request->trn_number;
        $company->tin_number = $request->tin_number;
        if ($request->hasFile('logo')) {
            $file = $request->file('logo');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/company/'), $filePath);
            $company-> logo = $filePath;
        }
        $company->status = 'active';
        $company->save();
        Session::flash('alert-success', 'Company created successfully');
        return redirect('/masters/companies');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function show(Company $company)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function edit($company_id)
    {
        $user = Auth::user();
        if (!$user->can('company-master')) {
            abort(403, 'Unauthorized action.');
        }

        $company = Company::find($company_id);
        return view('user.masters.companies.edit', compact('company'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $company_id)
    {
        $company = Company::find($company_id);
        $company->name = $request->name;
        $company->email = $request->email;
        session()->get('branch')->address = $request->address;
        $company->phone = $request->phone;
        $company->mobile = $request->mobile;
        $company->whatsapp = $request->whatsapp;
        $company->vat_number = $request->vat_number;
        $company->trn_number = $request->trn_number;
        $company->tin_number = $request->tin_number;
        if ($request->hasFile('logo')) {
            $file = $request->file('logo');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/company/'), $filePath);
            $company-> logo = $filePath;
        }
        $company->save();
        Session::flash('alert-success', 'Company updated successfully');
        return redirect('/masters/companies');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function destroy($company_id)
    {
        $company = Company::findOrFail($company_id);
        $company->delete();
        Session::flash('alert-success', 'Company deleted successfully');
        return redirect('/masters/companies');
    }
}
