<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
use App\PeriodClosing;

class PeriodClosingController extends Controller
{

    protected $utils;

    public function __construct(Utils $utils)
    {
        $this->utils = $utils;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('period-closing')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $period_closing = PeriodClosing::Where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(25);
        return view('user.masters.period_closing.index', compact('period_closing'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('period-closing')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.masters.period_closing.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $location_id = $this->getLocationId();

        $period_closing = new PeriodClosing();
        $period_closing->location_id = $location_id;
        $period_closing->from_date = $request->from_date;
        $period_closing->to_date = $request->to_date;
        $period_closing->carry_opening_balance = $request->has('carry_opening_balance') ? 1 : 0;
        $period_closing->save();
        Session::flash('alert-success', 'Period Closing created successfully');
        return redirect('/masters/period-closing');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\PeriodClosing  $period_closing
     * @return \Illuminate\Http\Response
     */
    public function show(PeriodClosing $period_closing)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\PeriodClosing  $period_closing
     * @return \Illuminate\Http\Response
     */
    public function edit($period_closing_id)
    {
        $user = Auth::user();
        if (!$user->can('period-closing')) {
            abort(403, 'Unauthorized action.');
        }

        $period_closing = PeriodClosing::find($period_closing_id);
        return view('user.masters.period_closing.edit', compact('period_closing'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\PeriodClosing  $period_closing
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $period_closing_id)
    {

        $period_closing = PeriodClosing::find($period_closing_id);
        $period_closing->from_date = $request->from_date;
        $period_closing->to_date = $request->to_date;
        $period_closing->carry_opening_balance = $request->has('carry_opening_balance') ? 1 : 0;
        $period_closing->save();
        Session::flash('alert-success', 'Period Closing updated successfully');
        return redirect('/masters/period-closing');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\PeriodClosing  $period_closing
     * @return \Illuminate\Http\Response
     */
    public function destroy($period_closing_id)
    {
        $period_closing = PeriodClosing::findOrFail($period_closing_id);
        $period_closing->delete();
        Session::flash('alert-success', 'Period Closing deleted successfully');
        return redirect('/masters/period-closing');
    }
}
