<?php

namespace App\Http\Controllers;

use App\Branches;
use Illuminate\Http\Request;
use App\ReceivedVoucher;
use App\MarketingDashboardImage;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Session;

class CrusherController extends Controller
{
    public function index()
    {
        $dashboardImage = MarketingDashboardImage::find(1);
        // ADMIN
        $branches = Branches::with('company','permissions.permission')->whereHas('company', function($q) {$q->where('user_id', session()->get('uv_user_id'));})->get();
        // SUBADMIN
        
        if(auth()->user()->role=='subadmin') 
        {
            $branches = auth()->user()->permissions->load('branchPermission.permission', 'branchPermission.branch');
            $branches = $branches->pluck('branchPermission.branch')->unique();
        }
        // Get the ID of the currently selected branch from the session
        $selectedBranchId = session('selected_branch');
        $expire = false;
        $leniencyExpire = false;
        $status = false;
        if (!empty($selectedBranchId)) 
        {
            $branch = Branches::find($selectedBranchId);
            session()->put('branch',$branch);
            $now = Carbon::parse(now())->format('Y-m-d');
            $branchExpireDate = Carbon::parse($branch->expiry_date);
            $leniency = $branch->leniency;
            if($branchExpireDate->addDays($leniency)->lt($now))
            {
                $expire = true;
                if (!$branch->vouchers->isEmpty()) 
                {
                    foreach ($branch->vouchers as $voucher) 
                    {
                        if ($voucher->created_at->gte($branchExpireDate)) 
                        {
                            $expire = false;
                        }
                    }
                }
            }

            $status = $branch->status;
        }

        // Find the selected branch within the fetched branches
        $selectedBranch = $branches->first(function ($branch) use ($selectedBranchId) {
            return $branch->id == $selectedBranchId;
        });
        $user = auth()->user();
        return view('crusher.index',compact('dashboardImage', 'branches', 'selectedBranch', 'user', 'expire', 'status'));
    }

    public function updateTermsAccept(Request $request)
    {
        // Get the authenticated user (you may need to modify this depending on your authentication setup)
        $user = Auth::user();

        // Update the terms_accept status to 1
        $user->update(['terms_accept' => 1]);

        return redirect()->back()->with('success', 'Terms and conditions accepted.');
    }

    public function upload(Request $request)
    {
        if ($request->hasFile("voucher")) {
            $file = $request->file('voucher');
            $fileName = time() . '-voucher-.' . $file->getClientOriginalExtension();
            $filePath = $file->storeAs('uploads/branch/vouchers/', $fileName, 'public');
            $data = ['branch_id'=>session('selected_branch'), 'voucher'=>$filePath];
            ReceivedVoucher::create($data);
            flash()->addSuccess('Uploaded');
            return back();
        }
        flash()->addError('Voucher is required');
        return back();
    }

    public function setSelectedBranch(Request $request)
    {
        $branchId = $request->input('branch');
        
        if ($branchId) {
            Session::put('selected_branch', $branchId);
        } else {
            Session::forget('selected_branch');
        }

        return redirect()->back();
    }
}
