<?php

namespace App\Http\Controllers\MainMenu;

use App\AccountMastersAccountsDetail;
use App\ClassOfAccount;
use App\Http\Controllers\Controller;
use App\ItemGroup;
use App\Location;
use App\PurchaseInvoice;
use App\PurchaseReturnVoucher;
use App\SaleInvoice;
use App\SalesReturnVoucher;
use Illuminate\Http\Request;
use Auth;

class ProfitLossController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('report-of-all-countries-locations')) {
            abort(403, 'Unauthorized action.');
        }

        // dd($request->all());
        $from_date = $request->from_date;
        $to_date = $request->to_date;
        $locations = Location::all();
        // $locations = Location::Where('name', "Muscat")->get();
        foreach ($locations as $location) {
            $location_id = $location->id;

            $sales_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('net_total');
            $sales_return_vouchers = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('net_total');
            $sales_accounts[] = $sales_invoices - $sales_return_vouchers;

            $purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('net_total');
            $purchase_return_vouchers = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('net_total');
            $purchase_accounts[] = $purchase_invoices - $purchase_return_vouchers;

            $groups = ItemGroup::with('item_names.purchase_sales_transfer_stock_tracks')->with('item_names.purchase_invoice')->where('location_id', $location_id)->get();

            $group_total_amount = 0;

            // $closing_amount = [];

            foreach ($groups as $group){
                foreach($group->item_names as $item_name){
                    foreach ($item_name->purchase_sales_transfer_stock_tracks as $key => $purchase_sales_transfer_stock_track){
                        $group_total_amount = $group_total_amount + ($purchase_sales_transfer_stock_track->quantity * $item_name->purchase_invoice[$key]->rate_per_quantity);
                    }
                }
            }

            $closing_amount[] =  $group_total_amount;

            $opening_sales_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('net_total');
            $opening_purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('net_total');
            $opening_sales_return_vouchers = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('net_total');
            $opening_purchase_return_vouchers = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('net_total');

            $opening_inward_amount = $opening_purchase_invoices + $opening_sales_return_vouchers;
            $opening_outward_amount = $opening_sales_invoices + $opening_purchase_return_vouchers;
            $opening_amount[] = $opening_inward_amount - $opening_outward_amount;

            $cost_of_sales[] = ( ($opening_inward_amount - $opening_outward_amount) + ($purchase_invoices - $purchase_return_vouchers) ) - ($group_total_amount);

            $gross_profit_loss[] = ($sales_invoices - $sales_return_vouchers) - ( ( ($opening_inward_amount - $opening_outward_amount) + ($purchase_invoices - $purchase_return_vouchers) ) - ($group_total_amount) );

            $trial_balance_closing_balance = [];
            $class_of_account = ClassOfAccount::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->where('name', "Expenses")->first();
            if ($class_of_account) {

                foreach ($class_of_account->account_master as $account_master){
                    $debit = 0;
                    $credit = 0;
                    $opening_balance = 0;
                    $closing_balance = 0;
                    $debit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('debit');
                    $credit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('credit');
                    $first_account = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $account_master->id)->first();
                    if($first_account){
                        $with_opening_balance_debit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('debit');
                        $with_opening_balance_credit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('credit');
                    }else{
                        $with_opening_balance_debit = 0;
                        $with_opening_balance_credit = 0;
                    }
                    $voucher_type_opening_balance = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('voucher_type', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->first();
                    if($voucher_type_opening_balance){
                        if($voucher_type_opening_balance-> debit > 0){
                            $opening_balance = $voucher_type_opening_balance-> debit;
                        }else{
                            $opening_balance = $voucher_type_opening_balance->credit;
                        }
                    }else {
                        if($account_master-> account_master_type == 'Supplier'){
                            $opening_balance = ($with_opening_balance_credit) - $with_opening_balance_debit;
                        }else{
                            $opening_balance = ($with_opening_balance_debit) - $with_opening_balance_credit;
                        }
                    }
                    if($account_master-> account_master_type == 'Supplier'){
                        $closing_balance = ($opening_balance + $credit) - $debit;
                    }else{
                        $closing_balance = ($opening_balance + $debit) - $credit;
                    }
                    $trial_balance_closing_balance[] = [
                        "ledger" => $account_master,
                        "closing_balance" => $closing_balance,
                    ];
                }

            }
            else {
                $trial_balance_closing_balance[] = [
                    "ledger" => "",
                    "closing_balance" => "",
                ];
            }

            $trial_balance_closing_balance_array[] =  $trial_balance_closing_balance;


            $expense_total = 0;
            foreach ($trial_balance_closing_balance as $closing) {
                $expense_total = $expense_total + $closing['closing_balance'];
            }

            $expense_total_array[] = $expense_total;

            $net_profit_loss[] = ( ($sales_invoices - $sales_return_vouchers) - ( ( ($opening_inward_amount - $opening_outward_amount) + ($purchase_invoices - $purchase_return_vouchers) ) - ($group_total_amount) ) ) - $expense_total;

        }
        // dd($closing_amount);


        return view('user.main_menu.profit_loss.index', compact('locations','from_date', 'to_date', 'sales_accounts', 'purchase_accounts', 'opening_amount', 'closing_amount', 'cost_of_sales', 'gross_profit_loss', 'trial_balance_closing_balance_array', 'expense_total_array', 'net_profit_loss'));
    }

    public function display()
    {
        $user = Auth::user();
        if (!$user->can('report-of-all-countries-locations')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.main_menu.profit_loss.select_date');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
