<?php

namespace App\Http\Controllers;

use App\AdminOtherAccount;
use App\AdminSalesInvoice;
use App\AdminSalesInvoiceExpense;
use App\AdminTermsAndCondition;
use App\AdminWarehouse;
use App\Company;
use App\Customer;
use App\DeliveryNote;
use App\Expense;
use App\ItemName;
use App\OtherAccountBuyer;
use App\Quotation;
use App\StaffAndSalesman;
use App\Supplier;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class AdminSalesInvoiceController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $salesInvoice = AdminSalesInvoice::with('account', 'item')
        ->whereIn('id', function ($query) {
            $query->selectRaw('MIN(id)')
                ->from('admin_sales_invoices')
                ->where('branch_id', session()->get('selected_branch'))
                ->whereNotNull('invoice_no')
                ->groupBy('invoice_no');
        })
        ->withTrashed()
        ->orderBy('id', 'desc')->get();

        return view('crusher.trans_sale_invoice.index', compact('salesInvoice'));
    }


    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $deliveryNotes = collect();
        $quotations = collect();

        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();
        $warehouses = AdminWarehouse::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'Expenses'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });
        $otherAccountExpenses = AdminOtherAccount::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account'=>'Expenses'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        $expensesAccount = $expenses->concat($otherAccountExpenses);

        $currentDate = Carbon::now()->format('Ymd');
        // Get the latest quotation number for the branch
        $latestQuotation = Quotation::where('branch_id', session()->get('selected_branch'))
            ->latest('id')->first();

        // Calculate the new quotation number
        if ($latestQuotation) {
            $lastQuotationNumber = explode('-', $latestQuotation->quotation_no)[1];
            $newQuotationNumber = $currentDate . '-' . str_pad((int)$lastQuotationNumber + 1, 3, '0', STR_PAD_LEFT);
        } else {
            $newQuotationNumber = $currentDate . '-001';
        }

        // Get the latest quotation number for the branch
        $latestDo = DeliveryNote::where('branch_id', session()->get('selected_branch'))
        ->latest('id')->first();

        // Calculate the new quotation number
        if ($latestDo) {
            $lastDoNumber = explode('-', $latestDo->do_no)[1];
            $newDoNumber = 'DO#'.$currentDate . '-' . str_pad((int)$lastDoNumber + 1, 3, '0', STR_PAD_LEFT);
        } else {
            $newDoNumber = 'DO#'.$currentDate . '-001';
        }

         // Get the latest quotation number for the branch
         $latestSi = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))
         ->latest('id')->first();

         // Calculate the new quotation number
         if ($latestSi) {
             $lastNumber = (int)substr($latestSi->invoice_no, -4); // Extract the last 4 digits
             $newSiNumber = str_pad($lastNumber + 1, 4, '0', STR_PAD_LEFT);
         } else {
             $newSiNumber = '0001';
         }

        $doNo = request()->input('do_no');

        if ($doNo) {
            $deliveryNotes = DeliveryNote::where('do_no',$doNo)->get();
            $quotations = Quotation::with('expenses')->where('quotation_no',$deliveryNotes->first()->quotation_no)->get();
        }

        $quotation = request()->input('quotation_no');

        if ($quotation) {
            $quotations = Quotation::with('expenses')->where('quotation_no',$quotation)->get();
        }

        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'sales_invoice'])->first();

         // Retrieve customers and include both 'account_name' and 'table_name' attributes
         $customers = Customer::where('branch_id', session()->get('selected_branch'))
         ->select('*', 'name_of_organisation as account_name')
         ->get()
         ->map(function ($item) {
             return $item->setAttribute('table_name', 'App\Customer');
         });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

         // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
         $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
         ->select('*', 'name_of_organisation as account_name')
         ->get()
         ->map(function ($item) {
             return $item->setAttribute('table_name', 'App\Supplier');
         });

         // Retrieve expenses and set 'table_name' to the fully-qualified model class name
         $expenses = Expense::where('branch_id', session()->get('selected_branch'))
         ->get()
         ->map(function ($item) {
             return $item->setAttribute('table_name', 'App\Expense');
         });

         // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
         $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
         ->get()
         ->map(function ($item) {
             return $item->setAttribute('table_name', 'App\AdminOtherAccount');
         });

         // Concatenate all collections into a single collection
         $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);

        //  dd($quotations);
        return view('crusher.trans_sale_invoice.create', compact('tnc', 'customers', 'items', 'quotations', 'newSiNumber', 'warehouses', 'expensesAccount', 'deliveryNotes', 'accounts'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */


     public function store(Request $request)
     {
         try {
             $validatedData = $request->validate([
                 'invoice_no' => 'nullable|string',
                 'quotation_no' => 'nullable|string',
                 'do_no' => 'nullable|string',
                 'lpo_no' => 'nullable|string',
                 'lpo_days' => 'nullable|string',
                 'term_of_payment' => 'nullable|string',
                 'narrations' => 'nullable|string',
                 'date' => 'nullable|date',
                 'account_id' => 'nullable',
                 'account_type' => 'nullable',
                 'item_description' => 'nullable|array',
                 'item_description.*' => 'nullable|string',
                 'item_id' => 'nullable|array',
                 'item_id.*' => 'nullable|exists:item_names,id',
                 'warehouse_id' => 'nullable|array',
                 'warehouse_id.*' => 'nullable|exists:admin_warehouses,id',
                 'quantity' => 'nullable|array',
                 'quantity.*' => 'nullable|numeric',
                 'rate_unit' => 'nullable|array',
                 'rate_unit.*' => 'nullable|numeric',
                 'gross_amount' => 'nullable|array',
                 'gross_amount.*' => 'nullable|numeric',
                 'vat_percentage' => 'nullable|array',
                 'vat_percentage.*' => 'nullable|numeric',
                 'vat_amount' => 'nullable|array',
                 'vat_amount.*' => 'nullable|numeric',
                 'net_amount' => 'nullable|array',
                 'net_amount.*' => 'nullable|numeric',
                 'expense_type' => 'nullable|array',
                 'expense_type.*' => 'nullable',
                 'expense_id' => 'nullable|array',
                 'expense_id.*' => 'nullable',
                 'expense_amount' => 'nullable|array',
                 'expense_amount.*' => 'nullable|numeric',
                 'total_quantity' => 'nullable|numeric',
                 'total_gross_amount' => 'nullable|numeric',
                 'total_vat_amount' => 'nullable|numeric',
                 'total_net_amount' => 'nullable|numeric',
                 'AmountInWords' => 'nullable|string',
                 'discount' => 'nullable',
                 'name' => 'nullable|string',
                 'phone' => 'nullable|string',
                 'email' => 'nullable|string',
                 'vat_number' => 'nullable|string',
                 'address' => 'nullable|string',
                 'type' => 'nullable|string',
             ]);

             $branch_id = session()->get('selected_branch');

              // Create an array to store buyer data
              $buyerData = [
                'name' => $validatedData['name'],
                'phone' => $validatedData['phone'],
                'email' => $validatedData['email'],
                'vat_number' => $validatedData['vat_number'],
                'address' => $validatedData['address'],
                'invoice_no' => $validatedData['invoice_no'],
                'type' => $validatedData['type'],
            ];

            // Check if buyer_account_id is present and create or update the buyer record
            if ($request->has('buyer_account_id')) {
                OtherAccountBuyer::updateOrCreate(['id' => $request->input('buyer_account_id')], $buyerData);
            } else {
                $buyer = OtherAccountBuyer::create($buyerData);
                $request->merge(['buyer_account_id' => $buyer->id]);
            }

             // Iterate over the item descriptions
             for ($i = 0; $i < count($validatedData['quantity']); $i++) {
                $invoiceData = [
                    'invoice_no' => $validatedData['invoice_no'],
                    'quotation_no' => $validatedData['quotation_no'],
                    'do_no' => $validatedData['do_no'],
                    'lpo_no' => $validatedData['lpo_no'],
                    'lpo_days' => $validatedData['lpo_days'],
                    'term_of_payment' => $validatedData['term_of_payment'],
                    'narrations' => $validatedData['narrations'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'account_type' => $validatedData['account_type'],
                    'item_description' => $validatedData['item_description'][$i] ?? (NULL),
                    'quantity' => $validatedData['quantity'][$i] ?? (NULL),
                    'rate_unit' => $validatedData['rate_unit'][$i] ?? (NULL),
                    'gross_amount' => $validatedData['gross_amount'][$i] ?? (NULL),
                    'vat_percentage' => $validatedData['vat_percentage'][$i] ?? (NULL),
                    'vat_amount' => $validatedData['vat_amount'][$i] ?? (NULL),
                    'net_amount' => $validatedData['net_amount'][$i] ?? (NULL),
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'AmountInWords' => $validatedData['AmountInWords'],
                    'discount' => $validatedData['discount'],
                    'branch_id' => $branch_id,
                    'item_id' => $validatedData['item_id'][$i] ?? (NULL),
                    'warehouse_id' => $validatedData['warehouse_id'][$i] ?? (NULL),
                ];

                $invoice = AdminSalesInvoice::create($invoiceData);


                if (isset($validatedData['expense_id']) && count($validatedData['expense_id']) > 0) {
                    for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
                        AdminSalesInvoiceExpense::create([
                            'admin_sales_invoice_id' => $invoice->id,
                            'expense_type' => $validatedData['expense_type'][$j] ?? null,
                            'expense_id' => $validatedData['expense_id'][$j] ?? null,
                            'expense_amount' => $validatedData['expense_amount'][$j] ?? null,
                        ]);
                    }
                }

        }

             flash()->addSuccess('Record Created');
             return redirect()->route('crusher.admin_sales_invoice.index');
         } catch (\Throwable $th) {
             flash()->addError($th->getMessage());
             return redirect()->back()->withInput();
         }
     }



    // public function store(Request $request)
    // {
    //     try {
    //         $validatedData = $request->validate([
    //             'invoice_no' => 'nullable|string',
    //             'quotation_no' => 'nullable|string',
    //             'do_no' => 'nullable|string',
    //             'lpo_no' => 'nullable|string',
    //             'lpo_days' => 'nullable|string',
    //             'term_of_payment' => 'nullable|string',
    //             'narrations' => 'nullable|string',
    //             'date' => 'nullable|date',
    //             'account_id' => 'nullable',
    //             'account_type' => 'nullable',
    //             'item_description' => 'nullable|array',
    //             'item_description.*' => 'nullable|string',
    //             'item_id' => 'nullable|array',
    //             'item_id.*' => 'nullable|exists:item_names,id',
    //             'warehouse_id' => 'nullable|array',
    //             'warehouse_id.*' => 'nullable|exists:ware_houses,id',
    //             'quantity' => 'nullable|array',
    //             'quantity.*' => 'nullable|numeric',
    //             'rate_unit' => 'nullable|array',
    //             'rate_unit.*' => 'nullable|numeric',
    //             'gross_amount' => 'nullable|array',
    //             'gross_amount.*' => 'nullable|numeric',
    //             'vat_percentage' => 'nullable|array',
    //             'vat_percentage.*' => 'nullable|numeric',
    //             'vat_amount' => 'nullable|array',
    //             'vat_amount.*' => 'nullable|numeric',
    //             'net_amount' => 'nullable|array',
    //             'net_amount.*' => 'nullable|numeric',
    //             'expense_type' => 'nullable|array',
    //             'expense_type.*' => 'nullable',
    //             'expense_id' => 'nullable|array',
    //             'expense_id.*' => 'nullable',
    //             'expense_amount' => 'nullable|array',
    //             'expense_amount.*' => 'nullable|numeric',
    //             'total_quantity' => 'nullable|numeric',
    //             'total_gross_amount' => 'nullable|numeric',
    //             'total_vat_amount' => 'nullable|numeric',
    //             'total_net_amount' => 'nullable|numeric',
    //             'discount' => 'nullable',
    //         ]);

    //         $branch_id = session()->get('selected_branch');

    //         // Iterate over the item descriptions
    //         for ($i = 0; $i < count($validatedData['quantity']); $i++) {
    //             $invoiceData = [
    //                 'invoice_no' => $validatedData['invoice_no'],
    //                 'quotation_no' => $validatedData['quotation_no'],
    //                 'do_no' => $validatedData['do_no'],
    //                 'lpo_no' => $validatedData['lpo_no'],
    //                 'lpo_days' => $validatedData['lpo_days'],
    //                 'term_of_payment' => $validatedData['term_of_payment'],
    //                 'narrations' => $validatedData['narrations'],
    //                 'date' => $validatedData['date'],
    //                 'account_id' => $validatedData['account_id'],
    //                 'account_type' => $validatedData['account_type'],
    //                 'item_description' => $validatedData['item_description'][$i] ?? (NULL),
    //                 'quantity' => $validatedData['quantity'][$i] ?? (NULL),
    //                 'rate_unit' => $validatedData['rate_unit'][$i] ?? (NULL),
    //                 'gross_amount' => $validatedData['gross_amount'][$i] ?? (NULL),
    //                 'vat_percentage' => $validatedData['vat_percentage'][$i] ?? (NULL),
    //                 'vat_amount' => $validatedData['vat_amount'][$i] ?? (NULL),
    //                 'net_amount' => $validatedData['net_amount'][$i] ?? (NULL),
    //                 'total_quantity' => $validatedData['total_quantity'],
    //                 'total_gross_amount' => $validatedData['total_gross_amount'],
    //                 'total_vat_amount' => $validatedData['total_vat_amount'],
    //                 'total_net_amount' => $validatedData['total_net_amount'],
    //                 'discount' => $validatedData['discount'],
    //                 'branch_id' => $branch_id,
    //                 'item_id' => $validatedData['item_id'][$i] ?? (NULL),
    //                 'warehouse_id' => $validatedData['warehouse_id'][$i] ?? (NULL),
    //             ];

    //             $invoice = AdminSalesInvoice::create($invoiceData);


    //             if (isset($validatedData['expense_id']) && count($validatedData['expense_id']) > 0) {
    //                 for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
    //                     AdminSalesInvoiceExpense::create([
    //                         'admin_sales_invoice_id' => $invoice->id,
    //                         'expense_type' => $validatedData['expense_type'][$j] ?? null,
    //                         'expense_id' => $validatedData['expense_id'][$j] ?? null,
    //                         'expense_amount' => $validatedData['expense_amount'][$j] ?? null,
    //                     ]);
    //                 }
    //             }

    //     }

    //         flash()->addSuccess('Record Created');
    //         return redirect()->route('crusher.admin_sales_invoice.index');
    //     } catch (\Throwable $th) {
    //         flash()->addError($th->getMessage());
    //         return redirect()->back()->withInput();
    //     }
    // }

    public function show(AdminSalesInvoice $adminSalesInvoice)
    {
        $salesInvoice = $adminSalesInvoice->with('account', 'item', 'warehouse', 'expenses', 'buyer')->where('invoice_no', $adminSalesInvoice->invoice_no)->get();
        // return $salesInvoice;
        $company = Company::where('user_id', auth()->id())->first();
        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'sales_invoice'])->first();
        return view('crusher.trans_sale_invoice.view', compact('salesInvoice', 'company', 'tnc'));
    }


    public function edit(AdminSalesInvoice $adminSalesInvoice)
    {
        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();
        $warehouses = AdminWarehouse::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'Expenses'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });
        $otherAccountExpenses = AdminOtherAccount::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account'=>'Expenses'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        $expensesAccount = $expenses->concat($otherAccountExpenses);

        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'sales_invoice'])->first();

         // Retrieve customers and include both 'account_name' and 'table_name' attributes
         $customers = Customer::where('branch_id', session()->get('selected_branch'))
         ->select('*', 'name_of_organisation as account_name')
         ->get()
         ->map(function ($item) {
             return $item->setAttribute('table_name', 'App\Customer');
         });

         // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
         $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
         ->select('*', 'name_of_organisation as account_name')
         ->get()
         ->map(function ($item) {
             return $item->setAttribute('table_name', 'App\Supplier');
         });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

         // Retrieve expenses and set 'table_name' to the fully-qualified model class name
         $expenses = Expense::where('branch_id', session()->get('selected_branch'))
         ->get()
         ->map(function ($item) {
             return $item->setAttribute('table_name', 'App\Expense');
         });

         // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
         $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
         ->get()
         ->map(function ($item) {
             return $item->setAttribute('table_name', 'App\AdminOtherAccount');
         });

         // Concatenate all collections into a single collection
         $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);


         $salesInvoices = $adminSalesInvoice->with('account', 'item', 'warehouse', 'expenses')->where('invoice_no', $adminSalesInvoice->invoice_no)->get();

        return view('crusher.trans_sale_invoice.edit', compact('tnc', 'items', 'warehouses', 'expensesAccount', 'accounts', 'salesInvoices'));
    }

    // public function update(Request $request, AdminSalesInvoice $adminSalesInvoice)
    // {
    //     try {
    //         $validatedData = $request->validate([
    //             'invoice_no' => 'nullable|string',
    //             'quotation_no' => 'nullable|string',
    //             'do_no' => 'nullable|string',
    //             'lpo_no' => 'nullable|string',
    //             'lpo_days' => 'nullable|string',
    //             'term_of_payment' => 'nullable|string',
    //             'narrations' => 'nullable|string',
    //             'date' => 'nullable|date',
    //             'account_id' => 'nullable',
    //             'account_type' => 'nullable',
    //             'item_description' => 'nullable|array',
    //             'item_description.*' => 'nullable|string',
    //             'item_id' => 'nullable|array',
    //             'item_id.*' => 'nullable|exists:item_names,id',
    //             'warehouse_id' => 'nullable|array',
    //             'warehouse_id.*' => 'nullable|exists:ware_houses,id',
    //             'quantity' => 'nullable|array',
    //             'quantity.*' => 'nullable|numeric',
    //             'rate_unit' => 'nullable|array',
    //             'rate_unit.*' => 'nullable|numeric',
    //             'gross_amount' => 'nullable|array',
    //             'gross_amount.*' => 'nullable|numeric',
    //             'vat_percentage' => 'nullable|array',
    //             'vat_percentage.*' => 'nullable|numeric',
    //             'vat_amount' => 'nullable|array',
    //             'vat_amount.*' => 'nullable|numeric',
    //             'net_amount' => 'nullable|array',
    //             'net_amount.*' => 'nullable|numeric',
    //             'expense_type' => 'nullable|array',
    //             'expense_type.*' => 'nullable',
    //             'expense_id' => 'nullable|array',
    //             'expense_id.*' => 'nullable',
    //             'expense_amount' => 'nullable|array',
    //             'expense_amount.*' => 'nullable|numeric',
    //             'total_quantity' => 'nullable|numeric',
    //             'total_gross_amount' => 'nullable|numeric',
    //             'total_vat_amount' => 'nullable|numeric',
    //             'total_net_amount' => 'nullable|numeric',
    //             'discount' => 'nullable',
    //         ]);

    //         $branch_id = session()->get('selected_branch');

    //         $adminSalesInvoice = $adminSalesInvoice->with('account', 'item', 'warehouse', 'expenses')->where('invoice_no', $adminSalesInvoice->invoice_no)->forceDelete();

    //         $invoice = [];  // Initialize the $invoice array before the loop

    //         // Iterate over the item descriptions
    //         for ($i = 0; $i < count($validatedData['quantity']); $i++) {
    //             $invoiceData = [
    //                 'invoice_no' => $validatedData['invoice_no'],
    //                 'quotation_no' => $validatedData['quotation_no'],
    //                 'do_no' => $validatedData['do_no'],
    //                 'lpo_no' => $validatedData['lpo_no'],
    //                 'lpo_days' => $validatedData['lpo_days'],
    //                 'term_of_payment' => $validatedData['term_of_payment'],
    //                 'narrations' => $validatedData['narrations'],
    //                 'date' => $validatedData['date'],
    //                 'account_id' => $validatedData['account_id'],
    //                 'account_type' => $validatedData['account_type'],
    //                 'item_description' => $validatedData['item_description'][$i] ?? null,
    //                 'quantity' => $validatedData['quantity'][$i] ?? null,
    //                 'rate_unit' => $validatedData['rate_unit'][$i] ?? null,
    //                 'gross_amount' => $validatedData['gross_amount'][$i] ?? null,
    //                 'vat_percentage' => $validatedData['vat_percentage'][$i] ?? null,
    //                 'vat_amount' => $validatedData['vat_amount'][$i] ?? null,
    //                 'net_amount' => $validatedData['net_amount'][$i] ?? null,
    //                 'total_quantity' => $validatedData['total_quantity'],
    //                 'total_gross_amount' => $validatedData['total_gross_amount'],
    //                 'total_vat_amount' => $validatedData['total_vat_amount'],
    //                 'total_net_amount' => $validatedData['total_net_amount'],
    //                 'discount' => $validatedData['discount'],
    //                 'branch_id' => $branch_id,
    //                 'item_id' => $validatedData['item_id'][$i] ?? null,
    //                 'warehouse_id' => $validatedData['warehouse_id'][$i] ?? null,
    //             ];

    //             $invoice[] = AdminSalesInvoice::create($invoiceData);  // Add each instance to the $invoice array
    //         }

    //         // corrected iteration for updating expenses
    //         if (isset($validatedData['expense_id']) && count($validatedData['expense_id']) > 0) {
    //             AdminSalesInvoiceExpense::where('admin_sales_invoice_id', $adminSalesInvoice->id)->forceDelete();

    //             for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
    //                 // Use the correct index to get the specific AdminSalesInvoice instance from the array
    //                 AdminSalesInvoiceExpense::create([
    //                     'admin_sales_invoice_id' => $invoice[$j]->id,  // Use $j to get the correct instance
    //                     'expense_type' => $validatedData['expense_type'][$j],
    //                     'expense_id' => $validatedData['expense_id'][$j],
    //                     'expense_amount' => $validatedData['expense_amount'][$j],
    //                 ]);
    //             }
    //         }

    //         flash()->addSuccess('Record Updated');
    //         return redirect()->route('crusher.admin_sales_invoice.index');
    //     } catch (\Throwable $th) {
    //         flash()->addError($th->getMessage());
    //         return redirect()->back()->withInput();
    //     }
    // }

    public function update(Request $request, AdminSalesInvoice $adminSalesInvoice)
    {
        try {
            $validatedData = $request->validate([
                'invoice_no' => 'nullable|string',
                'quotation_no' => 'nullable|string',
                'do_no' => 'nullable|string',
                'lpo_no' => 'nullable|string',
                'lpo_days' => 'nullable|string',
                'term_of_payment' => 'nullable|string',
                'narrations' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable',
                'account_type' => 'nullable',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'item_id' => 'nullable|array',
                'item_id.*' => 'nullable|exists:item_names,id',
                'warehouse_id' => 'nullable|array',
                'warehouse_id.*' => 'nullable|exists:admin_warehouses,id',
                'quantity' => 'nullable|array',
                'quantity.*' => 'nullable|numeric',
                'rate_unit' => 'nullable|array',
                'rate_unit.*' => 'nullable|numeric',
                'gross_amount' => 'nullable|array',
                'gross_amount.*' => 'nullable|numeric',
                'vat_percentage' => 'nullable|array',
                'vat_percentage.*' => 'nullable|numeric',
                'vat_amount' => 'nullable|array',
                'vat_amount.*' => 'nullable|numeric',
                'net_amount' => 'nullable|array',
                'net_amount.*' => 'nullable|numeric',
                'expense_type' => 'nullable|array',
                'expense_type.*' => 'nullable',
                'expense_id' => 'nullable|array',
                'expense_id.*' => 'nullable',
                'expense_amount' => 'nullable|array',
                'expense_amount.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'total_gross_amount' => 'nullable|numeric',
                'total_vat_amount' => 'nullable|numeric',
                'total_net_amount' => 'nullable|numeric',
                'discount' => 'nullable',
            ]);

            $branch_id = session()->get('selected_branch');

            $adminSalesInvoice->with('account', 'item', 'warehouse', 'expenses')->where('invoice_no', $adminSalesInvoice->invoice_no)->forceDelete();

            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['quantity']); $i++) {
                $invoiceData = [
                    'invoice_no' => $validatedData['invoice_no'],
                    'quotation_no' => $validatedData['quotation_no'],
                    'do_no' => $validatedData['do_no'],
                    'lpo_no' => $validatedData['lpo_no'],
                    'lpo_days' => $validatedData['lpo_days'],
                    'term_of_payment' => $validatedData['term_of_payment'],
                    'narrations' => $validatedData['narrations'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'account_type' => $validatedData['account_type'],
                    'item_description' => $validatedData['item_description'][$i] ?? null,
                    'quantity' => $validatedData['quantity'][$i] ?? null,
                    'rate_unit' => $validatedData['rate_unit'][$i] ?? null,
                    'gross_amount' => $validatedData['gross_amount'][$i] ?? null,
                    'vat_percentage' => $validatedData['vat_percentage'][$i] ?? null,
                    'vat_amount' => $validatedData['vat_amount'][$i] ?? null,
                    'net_amount' => $validatedData['net_amount'][$i] ?? null,
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'discount' => $validatedData['discount'],
                    'branch_id' => $branch_id,
                    'item_id' => $validatedData['item_id'][$i] ?? null,
                    'warehouse_id' => $validatedData['warehouse_id'][$i] ?? null,
                ];

                $invoice = AdminSalesInvoice::create($invoiceData);

                AdminSalesInvoiceExpense::where('admin_sales_invoice_id', $adminSalesInvoice->id)->forceDelete();
                // Add expenses for this invoice
                if (isset($validatedData['expense_id']) && count($validatedData['expense_id']) > 0) {
                    for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
                        AdminSalesInvoiceExpense::create([
                            'admin_sales_invoice_id' => $invoice->id,
                            'expense_type' => $validatedData['expense_type'][$j] ?? null,
                            'expense_id' => $validatedData['expense_id'][$j] ?? null,
                            'expense_amount' => $validatedData['expense_amount'][$j] ?? null,
                        ]);
                    }
                }
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('crusher.admin_sales_invoice.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function destroy(Quotation $quotation)
    {
        //
    }

    public function softDelete($saleInvoice)
    {
        $saleInvoice = AdminSalesInvoice::findOrFail($saleInvoice);
        $saleInvoice->where('invoice_no', $saleInvoice->invoice_no)->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.admin_sales_invoice.index');
    }

    function buyer(Request $request)
    {
        // return $request->all();
        try {
            unset($request['buyer_account_id']);
            unset($request['_token']);
            OtherAccountBuyer::firstOrCreate(['type'=>$request->type, 'invoice_no'=>$request->invoice_no],$request->all());
            return response()->json(['status'=>true]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }

    function buyerExists(Request $request)
    {
        // return $request->all();
        try {
            $buyer = OtherAccountBuyer::where(['type'=>$request->type, 'invoice_no'=>$request->invoice_no])->exists();
            return response()->json(['status'=>$buyer]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }

    function buyerDelete(Request $request)
    {
        // return $request->all();
        try {
            $buyer = OtherAccountBuyer::where(['type'=>$request->type, 'invoice_no'=>$request->invoice_no])->delete();
            return response()->json(['status'=>$buyer]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }
}
