<?php

namespace App\Http\Controllers\Transaction;
use App\Http\Controllers\Controller;
use App\ApprovalVoucher;
use App\ApprovalVoucherItem;
use Illuminate\Http\Request;

use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Supplier;
use App\AccountMaster;
use App\AccountMasterTrack;
use App\Company;
use App\Services\Utils;
use App\Services\Slug;
use App\WareHouse;
use App\ItemName;
use App\TermsAndConditionsApprovalVoucher;
use App\PurchaseInvoice;
use App\PurchaseSalesTransferStockTrack;
use App\WareHouseTrack;
use Illuminate\Support\Facades\Auth as FacadesAuth;

class ApprovalVoucherController extends Controller
{
    protected $utils;
    protected $slug;

    public function __construct(Utils $utils, Slug $slug)
    {
        $this->utils = $utils;
        $this->slug = $slug;
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('approval-voucher-delivery-notes')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $approval_vouchers = ApprovalVoucher::withTrashed()->with('account')->where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(25);
        return view('user.transactions.approval_voucher.index', compact('approval_vouchers'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('approval-voucher-delivery-notes')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $latest_invoice = ApprovalVoucher::where('location_id', $location_id)->withTrashed()->latest()->first();
        if($latest_invoice){
            $invoice_no = $latest_invoice-> invoice_no + 1;
        }else{
            $invoice_no = 1;
        }

        $warehouses = WareHouseTrack::with(['warehouse' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $item_names = ItemName::all();
        // $items = [];
        // $purchase_invoices = PurchaseInvoice::with('items')->where('location_id', $location_id)->get();
        // foreach ($purchase_invoices as $key => $purchase_invoice) {
        //     foreach ($purchase_invoice->items as $key => $item) {
        //         $items[] = $item;
        //     }
        // }
        $items = PurchaseSalesTransferStockTrack::Where('location_id', $location_id)->Where('quantity', '!=', 0)->get();
        $terms_and_conditions_approval_voucher = TermsAndConditionsApprovalVoucher::where('location_id', $location_id)->first();
        return view('user.transactions.approval_voucher.create', compact('invoice_no', 'account_masters', 'warehouses','item_names', 'items', 'terms_and_conditions_approval_voucher'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $approval_voucher = new ApprovalVoucher();
            $approval_voucher->location_id = $location_id;
            $approval_voucher->invoice_no = $request->invoice_no;
            $approval_voucher->date = $request->date;
            $approval_voucher->type = $request->type;
            // $approval_voucher->warehouse_id = $request->warehouse_id;
            $approval_voucher->account_id = $request->account_id;
            $approval_voucher->total_quantity = $request->total_quantity;
            $approval_voucher->customer_id = 1;
            $approval_voucher->narrations = $request->narrations;
            $approval_voucher->buyer_name = $request->buyer_name;
            $approval_voucher->buyer_mobile = $request->buyer_mobile;
            $approval_voucher->buyer_email = $request->buyer_email;
            $approval_voucher->buyer_address = $request->buyer_address;
            $approval_voucher->buyer_vat_number = $request->buyer_vat_number;
            $approval_voucher->is_signed = 0;
            $approval_voucher->save();

            foreach ($request->imei_serial_no as $key => $imei_serial_no) {
                $approval_voucher_item = new ApprovalVoucherItem();
                $approval_voucher_item-> approval_voucher_id = $approval_voucher-> id;
                $approval_voucher_item-> item_name_id = $request->item_name_id[$key];
                $approval_voucher_item-> item_unit_id = $request->item_unit[$key];
                $approval_voucher_item-> imei_serial_no = $request->imei_serial_no[$key];
                $approval_voucher_item-> warehouse_id = $request->warehouse_id[$key];
                $approval_voucher_item-> quantity = $request->quantity[$key];
                $approval_voucher_item-> remarks = $request->remarks[$key];
                $approval_voucher_item-> save();
            }
            DB::commit();
            Session::flash('alert-success','Approval Voucehr created successfully');
            return redirect('/transactions/approval-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\ApprovalVoucher  $approvalVoucher
     * @return \Illuminate\Http\Response
     */
    public function show(ApprovalVoucher $approvalVoucher)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\ApprovalVoucher  $approvalVoucher
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = Auth::user();
        if (!$user->can('edit-approval-voucher-delivery-notes')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $approval_voucher = ApprovalVoucher::with('items')->where('id', $id)->first();

        $account_id = $approval_voucher->account_id;
        $account = AccountMaster::Find($account_id);

        if ($account->transactions_password != "") {
            $pass_check = "yes";
        }elseif ($account->transactions_password == "") {
            $pass_check = "no";
        }

        $warehouses = WareHouseTrack::with(['warehouse' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $item_names = ItemName::all();
        // $purchase_invoices = PurchaseInvoice::with('items')->where('location_id', $location_id)->get();
        // foreach ($purchase_invoices as $key => $purchase_invoice) {
        //     foreach ($purchase_invoice->items as $key => $item) {
        //         $items[] = $item;
        //     }
        // }
        $items = PurchaseSalesTransferStockTrack::Where('location_id', $location_id)->Where('quantity', '!=', 0)->get();
        $terms_and_conditions_approval_voucher = TermsAndConditionsApprovalVoucher::where('location_id', $location_id)->first();
        return view('user.transactions.approval_voucher.edit', compact('approval_voucher', 'account_masters', 'warehouses','item_names', 'items', 'terms_and_conditions_approval_voucher', 'pass_check', 'account'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\ApprovalVoucher  $approvalVoucher
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request,$id)
    {

        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $approval_voucher = ApprovalVoucher::where('id', $id)->first();
            $approval_voucher->location_id = $location_id;
            $approval_voucher->invoice_no = $request->invoice_no;
            $approval_voucher->date = $request->date;
            $approval_voucher->type = $request->type;
            // $approval_voucher->warehouse_id = $request->warehouse_id;
            $approval_voucher->account_id = $request->account_id;
            $approval_voucher->total_quantity = $request->total_quantity;
            $approval_voucher->customer_id = 1;
            $approval_voucher->narrations = $request->narrations;
            $approval_voucher->buyer_name = $request->buyer_name;
            $approval_voucher->buyer_mobile = $request->buyer_mobile;
            $approval_voucher->buyer_email = $request->buyer_email;
            $approval_voucher->buyer_address = $request->buyer_address;
            $approval_voucher->buyer_vat_number = $request->buyer_vat_number;
            $approval_voucher->save();

            ApprovalVoucherItem::where('approval_voucher_id', $id)->forcedelete();

            foreach ($request->imei_serial_no as $key => $imei_serial_no) {
                $approval_voucher_item = new ApprovalVoucherItem();
                $approval_voucher_item-> approval_voucher_id = $approval_voucher-> id;
                $approval_voucher_item-> item_name_id = $request->item_name_id[$key];
                $approval_voucher_item-> item_unit_id = $request->item_unit[$key];
                $approval_voucher_item-> imei_serial_no = $request->imei_serial_no[$key];
                $approval_voucher_item-> warehouse_id = $request->warehouse_id[$key];
                $approval_voucher_item-> quantity = $request->quantity[$key];
                $approval_voucher_item-> remarks = $request->remarks[$key];
                $approval_voucher_item-> save();
            }
            DB::commit();
            Session::flash('alert-success','Approval Voucehr updated successfully');
            return redirect('/transactions/approval-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\ApprovalVoucher  $approvalVoucher
     * @return \Illuminate\Http\Response
     */
    public function destroy(ApprovalVoucher $approvalVoucher)
    {
        //
    }

    public function view($id)
    {
        $user = Auth::user();
        if (!$user->can('view-print-approval-voucher-delivery-notes')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $approval_voucher = ApprovalVoucher::withTrashed()->with('items_cancelled.name')->where('id', $id)->first();
        $buyer = AccountMaster::Where('id', $approval_voucher->account_id)->first();
        $company = Company::where('location_id', $location_id)->first();
        $warehouses = WareHouse::all();

        $account_masters = [];
        $account_master_tracks = AccountMasterTrack::Where('user_id', auth()->user()->id)->get();
        foreach ($account_master_tracks as $key => $account_master_track){
            $account_masters[] = AccountMaster::Where('location_id', $location_id)->Where('id', $account_master_track->account_master_id)->first();
        }

        $item_names = ItemName::all();
        $terms_and_conditions = TermsAndConditionsApprovalVoucher::where('location_id', $location_id)->first();

        return view('user.print.approval_voucher.index', compact('buyer', 'approval_voucher', 'company', 'warehouses', 'account_masters','item_names', 'terms_and_conditions'));
    }

    // public function view_signature($id)
    // {
    //     $user = AccountMaster::Where('id', $id)->first();
    //     $sign = [];
    //     $sign[0] = $user->authorised_sign_1_sign_sample ;
    //     $sign[1] = $user->authorised_sign_2_sign_sample ;
    //     $sign[2] = $user->authorised_sign_3_sign_sample ;

    //     return view('user.print.approval_voucher.view_sign', compact('sign'));
    // }

    public function softdelete($id)
    {
        $user = Auth::user();
        if (!$user->can('cancel-approval-voucher-delivery-notes')) {
            abort(403, 'Unauthorized action.');
        }

        $approval_voucher = ApprovalVoucher::Find($id);
        $approval_voucher->delete();
        $approval_voucher_items = ApprovalVoucherItem::Where('approval_voucher_id', $id)->get();
        foreach ($approval_voucher_items as $item) {
            $item->delete();
        }
        Session::flash('alert-success','Approval Voucehr Cancelled successfully');
        return redirect('/transactions/approval-vouchers');
    }

    public function signature($id){
        $user = Auth::user();
        if (!$user->can('approval-voucher-delivery-notes')) {
            abort(403, 'Unauthorized action.');
        }

        $approval_voucher = ApprovalVoucher::Find($id);
        return view('user.transactions.approval_voucher.signature', compact('approval_voucher'));
    }

    public function signatureUpload(Request $request, $id){
        $approval_voucher = ApprovalVoucher::Find($id);
        $folderPath = public_path('/user_files/img/client_signs/');
        $image_parts = explode(";base64,", $request->signed);
        $image_type_aux = explode("image/", $image_parts[0]);
        $image_type = $image_type_aux[1];
        $image_base64 = base64_decode($image_parts[1]);
        $file_name = 'approvalvoucher-' . $id . '.'.$image_type;
        $file = $folderPath . $file_name;
        file_put_contents($file, $image_base64);

        $approval_voucher->is_signed = 1;
        $approval_voucher->signature_path = $file_name;
        $approval_voucher->save();

        Session::flash('alert-success','Approval Voucher Signature Uploaded successfully');
        return redirect('/transactions/approval-vouchers');
    }
}
