<?php

namespace App\Http\Controllers\Transaction;

use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use App\ClassOfAccount;
use App\Currency;
use App\Http\Controllers\Controller;
use App\DepositWithdrawVoucher;
use App\DepositWithdrawVoucherItem;
use Illuminate\Http\Request;
use DB;
use Auth;
use Session;
use Carbon\Carbon;
use App\Supplier;
use App\Services\Utils;
use App\Services\Slug;
use App\WareHouse;

class DepositWithdrawVoucherController extends Controller
{
    protected $utils;
    protected $slug;

    public function __construct(Utils $utils, Slug $slug)
    {
        $this->utils = $utils;
        $this->slug = $slug;
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('deposit-and-withdraw')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $deposit_withdraw_vouchers = DepositWithdrawVoucher::withTrashed()->where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(100);
        return view('user.transactions.deposit_withdraw.index', compact('deposit_withdraw_vouchers'));

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('deposit-and-withdraw')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $latest_voucher = DepositWithdrawVoucher::where('location_id', $location_id)->withTrashed()->latest()->first();
        if($latest_voucher){
            $invoice_no = $latest_voucher-> invoice_no + 1;
        }else{
            $invoice_no = 1;
        }
        $bank_class = ClassOfAccount::Where('name', "Bank Account")->first();
        $cash_class = ClassOfAccount::Where('name', "Cash in Hand")->first();
        $bank_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $bank_class->id)->get();
        $cash_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $cash_class->id)->get();
        // dd($bank_accounts);
        $currency = Currency::Where('location_id', $location_id)->first();
        return view('user.transactions.deposit_withdraw.create', compact('invoice_no','currency','cash_accounts','bank_accounts'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request->all());
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $deposit_withdraw_voucher = new DepositWithdrawVoucher();
            $deposit_withdraw_voucher->location_id = $location_id;
            $deposit_withdraw_voucher->invoice_no = $request->invoice_no;
            $deposit_withdraw_voucher->date = $request->date;
            $deposit_withdraw_voucher->deposit_withdraw_option = $request->deposit_withdraw_option;
            if($request->deposit_withdraw_option == 'deposit'){
                $deposit_withdraw_voucher->cash_account = $request->deposit_cash_account;
                $deposit_withdraw_voucher->bank_account = $request->deposit_bank_account;
                $deposit_withdraw_voucher->cheaque_no = $request->deposit_cheaque_no;
                $deposit_withdraw_voucher->cheaque_date = $request->deposit_cheaque_date;
            }else if($request->deposit_withdraw_option == 'withdrawal'){
                $deposit_withdraw_voucher->cash_account = $request->withdrawal_cash_account;
                $deposit_withdraw_voucher->bank_account = $request->withdrawal_bank_account;
                $deposit_withdraw_voucher->cheaque_no = $request->withdrawal_cheaque_no;
                $deposit_withdraw_voucher->cheaque_date = $request->withdrawal_cheaque_date;
            }
            $deposit_withdraw_voucher->person_id = $request->person_id;
            $deposit_withdraw_voucher->person_name = $request->person_name;
            $deposit_withdraw_voucher->total_amount = $request-> total_amount;
            $deposit_withdraw_voucher->amount_in_words = 'amount in words';
            $deposit_withdraw_voucher->narrations =$request->narrations;
            $deposit_withdraw_voucher->save();


            if($request->deposit_withdraw_option == 'deposit'){
                $account_cash_deposit = new AccountMastersAccountsDetail();
                $account_cash_deposit->account_master_id = $request->deposit_cash_account;
                $account_cash_deposit->debit = 0;
                $account_cash_deposit->credit = $request->amount;
                $account_cash_deposit->date = $request->date;
                $account_cash_deposit->voucher_type = "deposit-withdraw";
                $account_cash_deposit->voucher_no = $request->invoice_no;
                $account_cash_deposit->save();

                $account_bank_deposit = new AccountMastersAccountsDetail();
                $account_bank_deposit->account_master_id = $request->deposit_bank_account;
                $account_bank_deposit->debit = $request->amount;
                $account_bank_deposit->credit = 0;
                $account_bank_deposit->date = $request->date;
                $account_bank_deposit->voucher_type = "deposit-withdraw";
                $account_bank_deposit->voucher_no = $request->invoice_no;
                $account_bank_deposit->save();
            }
            elseif($request->deposit_withdraw_option == 'withdrawal'){
                $account_bank_withdraw = new AccountMastersAccountsDetail();
                $account_bank_withdraw->account_master_id = $request->withdrawal_bank_account;
                $account_bank_withdraw->debit = 0;
                $account_bank_withdraw->credit = $request->amount;
                $account_bank_withdraw->date = $request->date;
                $account_bank_withdraw->voucher_type = "deposit-withdraw";
                $account_bank_withdraw->voucher_no = $request->invoice_no;
                $account_bank_withdraw->save();

                $account_cash_withdraw = new AccountMastersAccountsDetail();
                $account_cash_withdraw->account_master_id = $request->withdrawal_cash_account;
                $account_cash_withdraw->debit = $request->amount;
                $account_cash_withdraw->credit = 0;
                $account_cash_withdraw->date = $request->date;
                $account_cash_withdraw->voucher_type = "deposit-withdraw";
                $account_cash_withdraw->voucher_no = $request->invoice_no;
                $account_cash_withdraw->save();
            }

            DB::commit();
            Session::flash('alert-success','Deposit Withdraw Voucher created successfully');
            return redirect('/transactions/deposit-withdraw-vouchers');

        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\DepositWithdrawVoucher  $depositWithdrawVoucher
     * @return \Illuminate\Http\Response
     */
    public function show(DepositWithdrawVoucher $depositWithdrawVoucher)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\DepositWithdrawVoucher  $depositWithdrawVoucher
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = Auth::user();
        if (!$user->can('edit-deposit-and-withdraw')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $deposit_withdraw_voucher = DepositWithdrawVoucher::with('items')->where('id', $id)->first();
        $currency = Currency::Where('location_id', $location_id)->first();
        $bank_class = ClassOfAccount::Where('name', "Bank Account")->first();
        $cash_class = ClassOfAccount::Where('name', "Cash in Hand")->first();
        $bank_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $bank_class->id)->get();
        $cash_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $cash_class->id)->get();
        return view('user.transactions.deposit_withdraw.edit', compact('currency','deposit_withdraw_voucher','bank_accounts','cash_accounts'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\DepositWithdrawVoucher  $depositWithdrawVoucher
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        // dd($request->all());
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $deposit_withdraw_voucher = DepositWithdrawVoucher::where('id', $id)->first();
            $deposit_withdraw_voucher->location_id = $location_id;
            $deposit_withdraw_voucher->invoice_no = $request->invoice_no;
            $deposit_withdraw_voucher->date = $request->date;
            $deposit_withdraw_voucher->deposit_withdraw_option = $request->deposit_withdraw_option;
            if($request->deposit_withdraw_option == 'deposit'){
                $deposit_withdraw_voucher->cash_account = $request->deposit_cash_account;
                $deposit_withdraw_voucher->bank_account = $request->deposit_bank_account;
                $deposit_withdraw_voucher->cheaque_no = $request->deposit_cheaque_no;
                $deposit_withdraw_voucher->cheaque_date = $request->deposit_cheaque_date;
            }else if($request->deposit_withdraw_option == 'withdrawal'){
                $deposit_withdraw_voucher->cash_account = $request->withdrawal_cash_account;
                $deposit_withdraw_voucher->bank_account = $request->withdrawal_bank_account;
                $deposit_withdraw_voucher->cheaque_no = $request->withdrawal_cheaque_no;
                $deposit_withdraw_voucher->cheaque_date = $request->withdrawal_cheaque_date;
            }
            $deposit_withdraw_voucher->person_id = $request->person_id;
            $deposit_withdraw_voucher->person_name = $request->person_name;
            $deposit_withdraw_voucher->total_amount = $request-> total_amount;
            $deposit_withdraw_voucher->amount_in_words = 'amount in words';
            $deposit_withdraw_voucher->narrations =$request->narrations;
            $deposit_withdraw_voucher->save();

            $accounts = AccountMastersAccountsDetail::Where('voucher_type', "deposit-withdraw")->Where('voucher_no', $deposit_withdraw_voucher->invoice_no)->get();
            foreach ($accounts as $account) {
                $account->forcedelete();
            }

            if($request->deposit_withdraw_option == 'deposit'){
                $account_cash_deposit = new AccountMastersAccountsDetail();
                $account_cash_deposit->account_master_id = $request->deposit_cash_account;
                $account_cash_deposit->debit = 0;
                $account_cash_deposit->credit = $request->amount;
                $account_cash_deposit->date = $request->date;
                $account_cash_deposit->voucher_type = "deposit-withdraw";
                $account_cash_deposit->voucher_no = $request->invoice_no;
                $account_cash_deposit->save();

                $account_bank_deposit = new AccountMastersAccountsDetail();
                $account_bank_deposit->account_master_id = $request->deposit_bank_account;
                $account_bank_deposit->debit = $request->amount;
                $account_bank_deposit->credit = 0;
                $account_bank_deposit->date = $request->date;
                $account_bank_deposit->voucher_type = "deposit-withdraw";
                $account_bank_deposit->voucher_no = $request->invoice_no;
                $account_bank_deposit->save();
            }
            elseif($request->deposit_withdraw_option == 'withdrawal'){
                $account_bank_withdraw = new AccountMastersAccountsDetail();
                $account_bank_withdraw->account_master_id = $request->withdrawal_bank_account;
                $account_bank_withdraw->debit = 0;
                $account_bank_withdraw->credit = $request->amount;
                $account_bank_withdraw->date = $request->date;
                $account_bank_withdraw->voucher_type = "deposit-withdraw";
                $account_bank_withdraw->voucher_no = $request->invoice_no;
                $account_bank_withdraw->save();

                $account_cash_withdraw = new AccountMastersAccountsDetail();
                $account_cash_withdraw->account_master_id = $request->withdrawal_cash_account;
                $account_cash_withdraw->debit = $request->amount;
                $account_cash_withdraw->credit = 0;
                $account_cash_withdraw->date = $request->date;
                $account_cash_withdraw->voucher_type = "deposit-withdraw";
                $account_cash_withdraw->voucher_no = $request->invoice_no;
                $account_cash_withdraw->save();
            }

            DB::commit();
            Session::flash('alert-success','Deposit Withdraw Voucher updated successfully');
            return redirect('/transactions/deposit-withdraw-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\DepositWithdrawVoucher  $depositWithdrawVoucher
     * @return \Illuminate\Http\Response
     */
    public function destroy(DepositWithdrawVoucher $depositWithdrawVoucher)
    {
        //
    }

    public function softdelete($id)
    {
        $user = Auth::user();
        if (!$user->can('cancel-deposit-and-withdraw')) {
            abort(403, 'Unauthorized action.');
        }

        $deposit_withdraw_voucher = DepositWithdrawVoucher::Find($id);

        $accounts = AccountMastersAccountsDetail::Where('voucher_type', "deposit-withdraw")->Where('voucher_no', $deposit_withdraw_voucher->invoice_no)->get();
        foreach ($accounts as $account) {
            $account->delete();
        }

        $deposit_withdraw_voucher->delete();

        Session::flash('alert-success','Deposit Withdraw Vouchers Cancelled successfully');
        return redirect('/transactions/deposit-withdraw-vouchers');
    }
}
